#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 13 15:55:39 2021

@author: anton
"""

from SLiCAP import *

prj = initProject('Active antenna T1 matrix solutions')

files = ['AB_1', 'AB_2', 'CD_1', 'CD_2']
# In all circuit files:
# signal source: V1, value = 0 if gainType='gain' or value = 1 if gainType='vi'
# R1 model = 'r' value = 'R_ell'
# The name of the input node of the amplifier is 'i'
# The name of the output node of the amplifier is 'o'

# Target values for the output impedance z_o, the source-to-load voltage 
# transfer 'A_v', the load resistance 'R_L' and the antenna capacitance 'CA':

CA  = 6.3e-12
z_o = 50
R_L = 50
Rs  = 5

# Definition of symbolic variables that will be used in expressions:
z_s = sp.sympify('1/s/C_A')
A, B, C, D, R_ell, C_A, R_s = sp.symbols('A, B, C, D, R_ell, C_A, R_s')

# The program
for fileName in files:
    A_v = 1
    #makeNetlist(fileName + '.asc', fileName)
    i1 = instruction()
    i1.setCircuit(fileName + '.cir')
    i1.setSimType('symbolic')
    i1.setGainType('gain')
    i1.setDataType('laplace')
    
    i1.setSource('V1')
    
    # determination of Vi
    # Vi is the transfer from V1 to the voltage V_i at node i
    i1.setDetector('V_i')
    result = i1.execute()
    V_i = result.laplace
    
    # determination of Ii
    # Ii is the transfer from V1 to the current through V1
    i1.setDetector('I_V1')
    result = i1.execute()
    I_i = -result.laplace
    
    # determination of Vo
    # Vo is the transfer from V1 to the voltage V_o at node o
    i1.setDetector('V_o')
    result = i1.execute()
    V_o = result.laplace
    
    # determination of Io
    # Io is the transfer from V1 to the current through R2
    i1.setDetector('I_R1')
    result = i1.execute()
    I_o = result.laplace
    
    htmlPage('Determination of T1 matrix parameters')
    head2html('Test circuit')
    img2html(fileName + '.svg', 600)
    
    head2html('T1 matrix of the device under test')
    text2html('The T1 matrix of the device under test is found as:')
    
    # Calculate the T1 parameters
    T_A = sp.simplify(sp.limit(V_i/V_o, R_ell, 'oo'))
    T_B = sp.simplify(sp.limit(V_i/I_o, R_ell, 0))
    T_C = sp.simplify(sp.limit(I_i/V_o, R_ell, 'oo'))
    T_D = sp.simplify(sp.limit(I_i/I_o, R_ell, 0))
    
    T1s = sp.Matrix([[T_A, T_B], [T_C, T_D]])
    eqn2html('T_1', T1s)
    
    text2html('The matrix equation for the two-port (DUT) is found as:')
    ViIi = sp.Matrix([[sp.Symbol('V_i')], [sp.Symbol('I_i')]])
    VoIo = sp.Matrix([[sp.Symbol('V_o')], [sp.Symbol('I_o')]])
    eqn2html(ViIi, T1s*VoIo)
    
    text2html('The source-to-load transfer is obtained as:')
    eqn2html('A_v', V_o)
    
    text2html('The input impedance is found as:')
    zi = sp.simplify((T_A*R_ell+T_B)/(T_C*R_ell + T_D))
    eqn2html('z_i', zi)
    
    text2html('The output impedance is found as:')
    zo = sp.simplify((T_D*z_s+T_B)/(T_C*z_s + T_A))
    eqn2html('z_o', zo)
    
    # Solve the transfer and the output impedance
    V_o = V_o.subs([(R_ell, R_L), (C_A, CA), (R_s, Rs)])
    zo  =  zo.subs([(R_ell, R_L), (C_A, CA), (R_s, Rs)])
    if fileName[0] == 'A':
        sols = sp.solve([V_o-A_v, zo-z_o])
        T_A = sols[A]
        T_B = sols[B]
    elif fileName[0] == 'C':
        A_v = -A_v
        sols = sp.solve([V_o-A_v, zo-z_o])
        T_C = T_C.subs(C, sols[C])
        T_D = T_D.subs(D, sols[D])

    # Define the numeric T1 matrix
    T1n = sp.Matrix([[T_A, T_B], [T_C, T_D]])
    head2html('Numeric values')
    text2html('The numeric values are obtained after solving the equations for $z_o$ and $A_v$ for the target values given below.')
    text2html('Output impedance $z_o$:')
    eqn2html('z_o', z_o)
    text2html('Source-to-load voltage transfer $A_v$:')
    eqn2html('A_v', A_v)
    text2html('Antenna capacitance $C_A$:')
    eqn2html('C_A', CA)
    
    if fileName == 'AB_1':
        # Determine the turn ratios for the transformers
        n_T1 = i1.getElementValue('T1').subs([(R_s, Rs), (B, T_B), (A, T_A)])
        n_T2 = i1.getElementValue('T2').subs([(R_s, Rs), (B, T_B), (A, T_A)])
        head3html('Turn ratios transformers')
        text2html('Value of the current sense resistor: $R_s=' + str(Rs) + '\\Omega$')
        text2html('Transformer T1:')
        eqn2html('n_T1', n_T1)
        text2html('Transformer T2:')
        eqn2html('n_T2', n_T2)
        
    elif fileName == 'AB_2':
        # Determine the values of R2, R3 and R4:
        R_2 = i1.getElementValue('R2').subs([(B, T_B), (A, T_A)])
        R_3 = i1.getElementValue('R3').subs([(B, T_B), (A, T_A)])
        R_4 = i1.getElementValue('R4').subs([(B, T_B), (A, T_A)])
        head3html('Resistor values:')
        text2html('The value $R_2$ of R2:')
        eqn2html('R_2', R_2)
        text2html('The value $R_3$ of R3:')
        eqn2html('R_3', R_3)
        text2html('The value $R_4$ of R4:')
        eqn2html('R_4', R_4)
        
    elif fileName == 'CD_1':
        C_2 = i1.getElementValue('C2').subs([(C, sols[C]), (D, sols[D]), (C_A, CA), (R_s, Rs)])
        C_3 = sols[C]
        head3html('Resistor values:')
        text2html('The value $R_s$ of R2:')
        eqn2html('R_s', Rs)
        head3html('Capacitor values:')
        text2html('The value $C_2$ of C2:')
        eqn2html('C_2', C_2)
        text2html('The value $C_3$ of C3:')
        eqn2html('C_3', C_3)
        
    elif fileName == 'CD_2':
        C_3 = sols[C]
        head3html('Resistor values:')
        text2html('The value $R_2$ of R2:')
        eqn2html('R_2', z_o)
        head3html('Capacitor values:')
        text2html('The value $C_3$ of C3:')
        eqn2html('C_3', C_3)
        
    text2html('The T1 matrix of the amplifier:')
    eqn2html('T_1', T1n)
        
    file2html(fileName + '.txt')
ini.htmlIndex = ('index.html')
htmlPage('Comparison')
file2html('comparison.txt')
csv2html('comparison.csv')